//********************************************************
//
// Example Application of BRD_xxx functions for ADC10X2G service.
//
// (C) InSys by Dorokhin Andrey Feb, 2006
//
//=********************************************************

#include	<windows.h>
#include	<conio.h>

#include	"brd.h"
#include	"extn_andor.h"
#include	"ctrlstrm.h"
#include	"ctrladc.h"
#include	"CtrlAdc10x2g.h"
#include	"ctrlsdram.h"

const U16 PLD_CFG_TAG	= 0x0500; // tag of PLD

void DisplayError(S32 status, TCHAR* cmd_str);
S32 GetDataFromSrv(BRD_Handle handle, BRD_ServList* srv, int idx);
S32 AdcSettings(BRD_Handle hADC);
S32 SdramSettings(BRD_Handle hADC, ULONG& bBuf_size);
S32 SdramInput(BRD_Handle hADC, PVOID pSig, ULONG bBufSize, int num_chan);
S32 SdramInputDMA(BRD_Handle hADC, int idx, ULONG bBufSize, int num_chan);
S32 FifoInput(BRD_Handle hADC, PVOID pSig, ULONG bBufSize, int num_chan);
S32 FifoInputDMA(BRD_Handle hADC, int idx, ULONG bBufSize, int num_chan);
S32 GetPtertrigData(BRD_Handle hADC, int num_chan);
void WriteDataFile(int idx, PVOID pBuf, ULONG nNumberOfBytes);

int isADS10x2G_ver2 = 0;

//=************************* main *************************
//=********************************************************
void main( int argc, char *argv[] )
{
	S32		status;

	BRD_displayMode(BRDdm_VISIBLE | BRDdm_CONSOLE); //     :     
	
	S32	DevNum;
	status = BRD_init("brd.ini", &DevNum); //  
//	BRD_Error* pErrInfo = NULL;
//	status = BRD_error(&pErrInfo); //    

	//   LID (   )
	BRD_LidList lidList;
	lidList.item = 10;
	lidList.pLID = new U32[10];
	status = BRD_lidList(lidList.pLID, lidList.item, &lidList.itemReal);

	BRD_Info	info;
	info.size = sizeof(info);
	BRD_Handle handle[10];

	for(ULONG iDev = 0; iDev < lidList.itemReal; iDev++)
	{
		BRD_getInfo(lidList.pLID[iDev], &info); //    
//		UINT DevID = info.boardType >> 16;
		UINT RevID = info.boardType & 0xff;
		if(RevID >= 0x20)
			isADS10x2G_ver2 = 1;
		printf("Board: %s, DevID = 0x%x, RevID = 0x%x, Bus = %d, Dev = %d, Slot = %d.\n", 
			info.name, info.boardType >> 16, info.boardType & 0xff, info.bus, info.dev, info.slot);
//		printf("Board: %s, DevID = 0x%x, RevID = 0x%x, BusType = %d, BaseMem = %x.\n", 
//			info.name, info.boardType >> 16, info.boardType & 0xff, info.busType, info.base[0]);
		handle[iDev] = BRD_open(lidList.pLID[iDev], BRDopen_EXCLUSIVE, NULL); //     
		if(handle[iDev] > 0)
		{
			BRD_PuList PuList[5];
			U32 ItemReal;
			status = BRD_puList(handle[iDev], PuList, 5, &ItemReal); //    
			if(ItemReal <= 5)
			{
				for(U32 j = 0; j < ItemReal; j++)
				{
					printf("Programable unit %d: %s, Id = %d, Code = %x, Attr = %x.\n",
									j, PuList[j].puDescription, PuList[j].puId, PuList[j].puCode, PuList[j].puAttr);
					if(PuList[j].puCode == PLD_CFG_TAG)
					{ //   
						U32	PldState;
						status = BRD_puState(handle[iDev], PuList[j].puId, &PldState);
						printf("BRD_puState: PldState = %d\n", PldState);
						if(!PldState)
						{//  
							printf("BRD_puLoad: loading...\r");
							if(RevID == 0x20)
								status = BRD_puLoad(handle[iDev], PuList[j].puId, "ads10x2g_v20.mcs", &PldState);
							else
								status = BRD_puLoad(handle[iDev], PuList[j].puId, "ads10x2g_v10.mcs", &PldState);
							if(BRD_errcmp(status, BRDerr_OK))
								printf("BRD_puLoad: load is OK\n");
							else
								if(BRD_errcmp(status, BRDerr_PLD_TEST_DATA_ERROR))
									printf("BRD_puLoad: data error starting test\n");
								else
									if(BRD_errcmp(status, BRDerr_PLD_TEST_ADDRESS_ERROR))
										printf("BRD_puLoad: address error starting test\n");
									else
										printf("BRD_puLoad: error loading\n");
						}
					}
				}
			}
			BRD_ServList srvList[10];
			status = BRD_serviceList(handle[iDev], 0, srvList, 10, &ItemReal);
			if(ItemReal <= 10)
			{
				for(U32 j = 0; j < ItemReal; j++)
				{
					printf("Service %d: %s, Attr = %X.\n",
									j, srvList[j].name, srvList[j].attr);
				}
			}
			U32 iSrv;
			for(iSrv = 0; iSrv < ItemReal; iSrv++)
			{
				if(!strcmp(srvList[iSrv].name, "ADC10X2G0"))
					status = GetDataFromSrv(handle[iDev], &srvList[iSrv], iDev * ItemReal + iSrv);
			}
			status = BRD_close(handle[iDev]); //   
			printf("BRD_close: OK\n");
		}
	}
	status = BRD_cleanup();
	printf("BRD_cleanup: OK\n");
	printf("Press any key for leaving program...\n");
	_getch();
}

void DisplayError(S32 status, TCHAR* cmd_str)
{
	S32	real_status = BRD_errext(status);
	TCHAR msg[255];
	switch(real_status)
	{
	case BRDerr_OK:
		sprintf_s(msg, "%s: BRDerr_OK\n", cmd_str);
		break;
	case BRDerr_BAD_MODE:
		sprintf_s(msg, "%s: BRDerr_BAD_MODE\n", cmd_str);
		break;
	case BRDerr_INSUFFICIENT_SERVICES:
		sprintf_s(msg, "%s: BRDerr_INSUFFICIENT_SERVICES\n", cmd_str);
		break;
	default:
		sprintf_s(msg, "%s: Unknown error\n", cmd_str);
		break;
	}
	printf(msg);
}

S32 GetDataFromSrv(BRD_Handle handle, BRD_ServList* srv, int idx)
{
	S32		status;
	U32 mode = BRDcapt_EXCLUSIVE;
	BRD_Handle hADC = BRD_capture(handle, 0, &mode, srv->name, 10000);
	if(mode == BRDcapt_EXCLUSIVE) printf("ADC Capture mode: EXCLUSIVE\n");
	if(mode == BRDcapt_SPY)	printf("ADC Capture mode: SPY\n");
	if(hADC > 0)
	{
		ULONG master = BRDims_SINGLE; //    () 
		status = BRD_ctrl(hADC, 0, BRDctrl_ADC_SETMASTER, &master);

		ULONG chan_mask = 1; //  
		status = BRD_ctrl(hADC, 0, BRDctrl_ADC_SETCHANMASK, &chan_mask);
		int num_chan = 1;

		status = AdcSettings(hADC); //   
		//Sleep(40);

		ULONG samplesOfChannel = 32000; //    
		ULONG bBufSize = (samplesOfChannel << 1) * num_chan; //      

		//    
		BRD_SdramCfg SdramConfig;
		ULONG PhysMemSize;
		status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_GETCFG, &SdramConfig);
		if(status < 0)
			PhysMemSize = 0;
		else
			PhysMemSize =	(1 << SdramConfig.RowAddrBits) *
							(1 << SdramConfig.ColAddrBits) * 
							SdramConfig.ModuleBanks * 
							SdramConfig.ChipBanks *
							SdramConfig.ModuleCnt * 2; //  32- 

		if(PhysMemSize)
		{ //     
			ULONG target = 2; //      
			status = BRD_ctrl(hADC, 0, BRDctrl_ADC_SETTARGET, &target);

			status = SdramSettings(hADC, bBufSize); //   SDRAM
			samplesOfChannel = (bBufSize >> 1) / num_chan;
			printf("Samples of channel = %d\n", samplesOfChannel);

			//       
			//PVOID pSig = new char[bBufSize];
			//status = SdramInput(hADC, pSig, bBufSize, num_chan); //   
			//WriteDataFile(idx, pSig, bBufSize);
			//delete pSig;

			//     - 
			status = SdramInputDMA(hADC, idx, bBufSize, num_chan); //   
		}
		else
		{
			 //   SDRAM (     BRDctrl_SDRAM_GETCFG,     )
			ULONG mem_size = 0;
			status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_SETMEMSIZE, &mem_size);

			//   FIFO    
			//PVOID pSig = new char[bBufSize];
			//status = FifoInput(hADC, pSig, bBufSize, num_chan); //   
			//WriteDataFile(idx, pSig, bBufSize);
			//delete pSig;

			//   FIFO  - 
			status = FifoInputDMA(hADC, idx, bBufSize, num_chan); //   
		}

		status = BRD_release(hADC, 0);
	}
	return status;
}

//   
S32 AdcSettings(BRD_Handle hADC)
{
	S32		status;

	//             
	BRD_SyncMode sync_mode;
	if(isADS10x2G_ver2)
	{
		sync_mode.clkSrc = BRDclks_ADC_INTREFPLL; //   -       
		sync_mode.clkValue = 1000000000.; // 1 GHz -  
		sync_mode.rate = 500000000.; // 500 MHz -  
	}
	else
	{
		sync_mode.clkSrc = BRDclks_ADC_MASTERCLK; //   -  
		sync_mode.clkValue = 1000000000.; // 1 GHz -  
		sync_mode.rate = 1000000000.; // 1 GHz -       ( )
	}
	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_SETSYNCMODE, &sync_mode);
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_ADC_SETSYNCMODE: clock = %f MHz, rate = %f MHz\n", sync_mode.clkValue / 1000000, sync_mode.rate / 1000000);
	else
		DisplayError(status, "BRDctrl_ADC_SETSYNCMODE");

	//       
	double rate = 1200000000.;
	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_SETRATE, &rate);
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_ADC_SETRATE: rate = %f MHz\n", rate / 1000000);
	else
		DisplayError(status, "BRDctrl_ADC_SETRATE");

	//         
	BRD_ClkMode clk_mode;
	if(isADS10x2G_ver2)
	{
		clk_mode.src = BRDclks_ADC_INTREFPLL; //   -       
		//clk_mode.src = BRDclks_ADC_EXTCLK;
		clk_mode.value = 1000000000.; // 1 GHz -  
	}
	else
	{
		clk_mode.src = BRDclks_ADC_MASTERCLK; //   -  
		clk_mode.value = 1000000000.; // 1 GHz -  
	}
	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_SETCLKMODE, &clk_mode);
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_ADC_SETCLKMODE: source = %d, value = %f MHz\n", clk_mode.src, clk_mode.value / 1000000);
	else
		DisplayError(status, "BRDctrl_ADC_SETCLKMODE");

	//    
	BRD_AdcStartMode start;
	start.src = BRDsts_ADC_PRG;
	start.inv = 0;
	start.level = 0.0;
	if(start.src != BRDsts_ADC_PRG)
		start.stndStart.startSrc = BRDsts_SUBMOD;
	else
		start.stndStart.startSrc = BRDsts_PRG;
	start.stndStart.startInv = 0;
	start.stndStart.trigOn = 1;
	start.stndStart.trigStopSrc = BRDsts_PRG;
	start.stndStart.stopInv = 0;
	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_SETSTARTMODE, &start);
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_ADC_SETSTARTMODE: start source = %d\n", start.src);
	else
		DisplayError(status, "BRDctrl_ADC_SETSTARTMODE");

	BRD_ValChan value_chan;
	value_chan.chan = 0;
	value_chan.value = 0.0;
	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_SETBIAS, &value_chan);
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_ADC_SETBIAS: bias = %f\n", value_chan.value);
	else
		DisplayError(status, "BRDctrl_ADC_SETBIAS");
	value_chan.value = 0.5;
	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_SETINPRANGE, &value_chan);
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_ADC_SETINPRANGE: range = %f\n", value_chan.value);
	else
		DisplayError(status, "BRDctrl_ADC_SETINPRANGE");
	value_chan.value = 1.;
	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_SETINPRESIST, &value_chan);
	if(!BRD_errcmp(status, BRDerr_OK))
		DisplayError(status, "BRDctrl_ADC_SETINPRESIST");
	value_chan.value = 1.;
	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_SETDCCOUPLING, &value_chan);
	if(!BRD_errcmp(status, BRDerr_OK))
		DisplayError(status, "BRDctrl_ADC_SETDCCOUPLING");

	return status;
}

//   SDRAM
S32 SdramSettings(BRD_Handle hADC, ULONG& bBuf_size)
{
	S32		status;

	ULONG mode = 0; //     
	status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_SETREADMODE, &mode);

	ULONG addr = 0;
	status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_SETSTARTADDR, &addr); //   
	status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_SETREADADDR, &addr); //   

	ULONG mem_size = bBuf_size >> 2; //      32- 
	status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_SETMEMSIZE, &mem_size);
	bBuf_size = mem_size << 2; //       
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_SDRAM_SETMEMSIZE: SDRAM buffer size = %d bytes\n", bBuf_size);
	else
		DisplayError(status, "BRDctrl_SDRAM_SETMEMSIZE");

	return status;
}

//     SDRAM      
S32 SdramInput(BRD_Handle hADC, PVOID pSig, ULONG bBufSize, int num_chan)
{
	S32		status;
	ULONG Status = 0;
	ULONG Enable = 1;

	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_FIFORESET, NULL); //  FIFO 
	status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_FIFORESET, NULL); //  FIFO SDRAM
	status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_ENABLE, &Enable); //    SDRAM
	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_ENABLE, &Enable); //   

	//   
	do {
//		status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_FIFOSTATUS, &Status);
		status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_ISACQCOMPLETE, &Status);
	} while(!Status);
//	} while(Status & 0x10);

	Enable = 0;
	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_ENABLE, &Enable); //   
	status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_ENABLE, &Enable); //    SDRAM

	//  ,    
	status = GetPtertrigData(hADC, num_chan);

	BRD_DataBuf data_buf;
	data_buf.pData = pSig;
	data_buf.size = bBufSize;
	status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_GETDATA, &data_buf);

	return status;
}

//     SDRAM  -   
S32 SdramInputDMA(BRD_Handle hADC, int idx, ULONG bBufSize, int num_chan)
{
	printf("DAQ into SDRAM\n");
	S32		status;
	ULONG Status = 0;
	ULONG Enable = 1;

	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_FIFORESET, NULL); //  FIFO 
	status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_FIFORESET, NULL); //  FIFO SDRAM
	status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_ENABLE, &Enable); //    SDRAM
	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_ENABLE, &Enable); //   

	//   
	do {
//		status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_FIFOSTATUS, &Status);
		status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_ISACQCOMPLETE, &Status);
	} while(!Status);
//	} while(Status & 0x10);

	Enable = 0;
	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_ENABLE, &Enable); //   
	status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_ENABLE, &Enable); //    SDRAM

	//  ,    
	status = GetPtertrigData(hADC, num_chan);

	BRDctrl_StreamCBufAlloc buf_dscr;
	buf_dscr.dir = BRDstrm_DIR_IN;
	buf_dscr.isCont = 0; //      
//	buf_dscr.isCont = 1; //      
	buf_dscr.blkNum = 1;
	buf_dscr.blkSize = bBufSize;
	buf_dscr.ppBlk = new PVOID[buf_dscr.blkNum];
	status = BRD_ctrl(hADC, 0, BRDctrl_STREAM_CBUF_ALLOC, &buf_dscr);

	// ,     
	ULONG tetrad;
	status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_GETSRCSTREAM, &tetrad);
	status = BRD_ctrl(hADC, 0, BRDctrl_STREAM_SETSRC, &tetrad);

	BRDctrl_StreamCBufStart start_pars;
	start_pars.isCycle = 0; //   
	status = BRD_ctrl(hADC, 0, BRDctrl_STREAM_CBUF_START, &start_pars);
	ULONG msTimeout = 20000; //      20 .
	status = BRD_ctrl(hADC, 0, BRDctrl_STREAM_CBUF_WAITBUF, &msTimeout);
	if(BRD_errcmp(status, BRDerr_WAIT_TIMEOUT))
	{	//    -,  
		status = BRD_ctrl(hADC, 0, BRDctrl_STREAM_CBUF_STOP, NULL);
		DisplayError(status, "TIME-OUT");
	}

	WriteDataFile(idx, buf_dscr.ppBlk[0], bBufSize);

	status = BRD_ctrl(hADC, 0, BRDctrl_STREAM_CBUF_FREE, &buf_dscr);
	delete[] buf_dscr.ppBlk;

	return status;
}

//     FIFO      
S32 FifoInput(BRD_Handle hADC, PVOID pSig, ULONG bBufSize, int num_chan)
{
	S32		status;
	ULONG Status = 0;
	ULONG Enable = 1;

	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_FIFORESET, NULL); //  FIFO 
	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_ENABLE, &Enable); //   

	//   FIFO
	do {
		status = BRD_ctrl(hADC, 0, BRDctrl_ADC_FIFOSTATUS, &Status);
	} while(Status & 0x40);

	Enable = 0;
	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_ENABLE, &Enable); //   

	BRD_DataBuf data_buf;
	data_buf.pData = pSig;
	data_buf.size = bBufSize;
	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_GETDATA, &data_buf);

	return status;
}

//     FIFO  -   
S32 FifoInputDMA(BRD_Handle hADC, int idx, ULONG bBufSize, int num_chan)
{
	printf("DAQ into FIFO\n");
	S32		status;
	ULONG Status = 0;

	BRDctrl_StreamCBufAlloc buf_dscr;
	buf_dscr.dir = BRDstrm_DIR_IN;
	buf_dscr.isCont = 0; //      
//	buf_dscr.isCont = 1; //      
	buf_dscr.blkNum = 1;
	buf_dscr.blkSize = bBufSize;
	buf_dscr.ppBlk = new PVOID[buf_dscr.blkNum];
	status = BRD_ctrl(hADC, 0, BRDctrl_STREAM_CBUF_ALLOC, &buf_dscr);

	//      
	ULONG flag = BRDstrm_DRQ_HALF; //   - FIFO  
	status = BRD_ctrl(hADC, 0, BRDctrl_STREAM_SETDRQ, &flag);

	//     
	ULONG tetrad;
	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_GETSRCSTREAM, &tetrad); //     
	status = BRD_ctrl(hADC, 0, BRDctrl_STREAM_SETSRC, &tetrad);

	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_FIFORESET, NULL); //  FIFO 

	BRDctrl_StreamCBufStart start_pars;
	start_pars.isCycle = 0; //   
	status = BRD_ctrl(hADC, 0, BRDctrl_STREAM_CBUF_START, &start_pars); //  
	ULONG Enable = 1;
	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_ENABLE, &Enable); //   
	ULONG msTimeout = 20000; //      20 .
	status = BRD_ctrl(hADC, 0, BRDctrl_STREAM_CBUF_WAITBUF, &msTimeout);
	if(BRD_errcmp(status, BRDerr_WAIT_TIMEOUT))
	{	//    -,  
		status = BRD_ctrl(hADC, 0, BRDctrl_STREAM_CBUF_STOP, NULL);
		DisplayError(status, "TIME-OUT");
	}
	Enable = 0;
	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_ENABLE, &Enable); //   

	WriteDataFile(idx, buf_dscr.ppBlk[0], bBufSize);

	status = BRD_ctrl(hADC, 0, BRDctrl_STREAM_CBUF_FREE, &buf_dscr);
	delete[] buf_dscr.ppBlk;

	return status;
}

//  ,    
S32 GetPtertrigData(BRD_Handle hADC, int num_chan)
{
	S32		status;
	//         
	ULONG start_event;
	status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_GETPRETRIGEVENT, &start_event);
	ULONG bStartEvent = start_event << 2; //   
	//   
	ULONG prec_event;
	status = BRD_ctrl(hADC, 0, BRDctrl_ADC_GETPREVENTPREC, &prec_event);
	ULONG bPrecEvent = prec_event << 1; //   
	bStartEvent += bPrecEvent * num_chan;
	//        
	ULONG isPassMemEnd;
	status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_ISPASSMEMEND, &isPassMemEnd);
	//     
	ULONG acq_size;
	status = BRD_ctrl(hADC, 0, BRDctrl_SDRAM_GETACQSIZE, &acq_size);
	ULONG bRealSize = acq_size << 2; //   

	return status;
}


void WriteDataFile(int idx, PVOID pBuf, ULONG nNumberOfBytes)
{
	TCHAR fileName[64];
	sprintf_s(fileName, "data_%d.bin", idx);
	HANDLE hfile = CreateFile(	fileName,
								GENERIC_WRITE,
								FILE_SHARE_WRITE,
								NULL,
								CREATE_ALWAYS,
								FILE_ATTRIBUTE_NORMAL,
								NULL);
	if(hfile == INVALID_HANDLE_VALUE)
	{
		printf("Create file error\n");
		return;
	}
	ULONG nNumberOfWriteBytes;

	BOOL bResult = WriteFile(hfile, pBuf, nNumberOfBytes, &nNumberOfWriteBytes, NULL);
	CloseHandle(hfile);
	if(bResult != TRUE)
		printf("Write file error\n");
}
//
// End of file
//
